"""
ArchiveDateGrep v1.0
Checks for files modifies after a given date time and then adds them to a archive 
file that can be easily extracted -

Philip Roche http://philroche.net/downloads
"""

import os, time, sys
import zipfile
import tarfile
import glob
from datetime import datetime


class ArchiveDateGrep:
    today = time.time()
    todaynow = datetime.now()
    yesterdayTimestamp = today-24*60*60
    yesterday = datetime.fromtimestamp(yesterdayTimestamp)
    iYear = yesterday.year
    iMonth = yesterday.month
    iDay = yesterday.day
    iHour = yesterday.hour
    iHourOrig = iHour
    iMin = yesterday.minute
    zFileName = ""
    logFileName = ""
    sExcludeFiles  =""
    bExcludeFiles = False
    lExcludeFiles = ['list.txt','log.txt']

   
    sYear,sMonth,sDay,sHour,sMin,sMode = "","","","","",""
            
    def __init__(self):
        self.GetArgs()
        self.ArchiveFiles()
        print "Finished"    
    
    def GetArgs(self):
        self.sMode=str(raw_input("Mode (zip|tar)[default: zip]: "))
        self.sMode = self.sMode.strip().lower()
        if self.sMode != "" and (self.sMode == "zip" or self.sMode == "tar"):
            try:
                self.sMode = self.sMode
            except:
                print "This is not a valid mode"
                self.sMode = "zip"
                #exception in assigning mode
        else:
            self.sMode = "zip"
                
        self.sYear=str(raw_input("Year (yyyy)[default: "+str(self.iYear)+"]: "))
        self.sYear = self.sYear.strip()
        if self.sYear != "":
            try:
                self.iYear = int(self.sYear)
            except:
                print "This is not a valid year"
                #exception in casting to int
            
        self.sMonth=str(raw_input("Month (mm)[default: "+str(self.iMonth)+"]: "))
        self.sMonth = self.sMonth.strip()
        if self.sMonth != "":
            try:
                self.iMonth = int(self.sMonth)
            except:
                print "This is not a valid month"
                #exception in casting to int
                
        self.sDay=str(raw_input("Day (dd)[default: "+str(self.iDay)+"]: "))
        self.sDay = self.sDay.strip()
        if self.sDay != "":
            try:
                self.iDay = int(self.sDay)
            except:
                print "This is not a valid day"
                #exception in casting to int
                
        self.sHour=str(raw_input("Hour (hh)[default: "+str(self.iHour)+"]: "))
        self.sHour = self.sHour.strip()
        if self.sHour != "":
            try:
                if int(self.sHour) < 25:
                    self.iHour = int(self.sHour)
            except:
                print "This is not a valid hour" + str(self.sHour)
                #exception in casting to int
                
        self.sMin=str(raw_input("Minute (mm)[default: "+str(self.iMin)+"]: "))
        self.sMin = self.sMin.strip()
        if self.sMin != "":
            try:
                if int(self.sMin) < 61:
                    self.iMin = int(self.sMin)
            except:
                print "This is not a valid minute" + str(self.sMin)
                #exception in casting to int
                

        
        self.sExcludeFiles = str(raw_input("Exclude Files (y|n)[default: n]: "))
        if self.sExcludeFiles.lower() == 'y':
            self.bExcludeFiles = True
        else:
            self.bExcludeFiles = False 
            
            
    def ArchiveFiles(self):
        '''
        0  	tm_year  	(for example, 1993)
        1 	tm_mon 	range [1,12]
        2 	tm_mday 	range [1,31]
        3 	tm_hour 	range [0,23]
        4 	tm_min 	range [0,59]
        5 	tm_sec 	range [0,61]; see (1) in strftime() description
        6 	tm_wday 	range [0,6], Monday is 0
        7 	tm_yday 	range [1,366]
        8 	tm_isdst 	0, 1 or -1; see below
        '''
        iBaseTime=time.mktime((self.iYear,self.iMonth,self.iDay,self.iHour,self.iMin,0,0,0,0))
        
        if  self.iHour != self.iHourOrig:
            iBaseTime = iBaseTime + time.altzone
        
        currentFileName = os.path.basename(str(sys.argv[0]))
        strTimeStamp = "-"+str(self.todaynow.hour)+"-"+str(self.todaynow.minute)+"-"+str(self.todaynow.day)+"-"+str(self.todaynow.month)+"-"+str(self.todaynow.year)
        if self.sMode == "tar":
            self.zFileName = "ArchiveDateGrep"+strTimeStamp+".tar"
            zFile = tarfile.open(self.zFileName, "w")
        else:
            self.zFileName = "ArchiveDateGrep"+strTimeStamp+".zip"
            zFile = zipfile.ZipFile(self.zFileName, "w")
                
        self.logFileName = "ArchiveDateGrep"+strTimeStamp+".log"
    
        f=open(self.logFileName,'w')
        
        
        f.write("- File changed since - %s\n"%time.asctime(time.localtime(iBaseTime)))
        
        #get a list of all the root level directories
        curDir = os.curdir
        
        for root,dirs,files in os.walk(curDir):
            if root.lower().find("svn")>-1 or root.lower().find("cvs")>-1 : 
                continue
            for name in files:
                if name.lower().find("svn") > -1 or name.lower().find("cvs")>-1 or name == self.zFileName or name == self.logFileName or name == currentFileName: 
                    continue
                iTime=os.path.getmtime(os.path.join(root,name))
                if iTime and iTime > iBaseTime: 
                    #add the file to the archive
                    if self.bExcludeFiles == False or (self.bExcludeFiles == True and root not in self.lExcludeFiles and name not in self.lExcludeFiles):  
                        curPath = os.path.join(root,name)   
                        curPath = curPath.replace(".\\","")
                        if curPath.find('.')==0:
                            curPath = curPath[1:]                    
                        print curPath
                        f.write("%s %s\n"%(time.asctime(time.localtime(iTime)),curPath))
                        if self.sMode == "tar":
                            zFile.add(curPath)
                        else:
                            zFile.write(curPath, curPath, zipfile.ZIP_DEFLATED)
        zFile.close()
        f.flush()
        f.close()
        
if __name__ == '__main__':
    ArchiveDateGrep = ArchiveDateGrep()
    
